-- ---------------------------------------------------
-- Tidy Plates: Neon - Theme Definition
-- ---------------------------------------------------

-- Start Theme
SetCVar("threatWarning", 3)
SetCVar("ShowClassColorInNameplate", 1)
SetCVar("CameraDistanceMaxFactor", 6)

local path = "Interface\\Addons\\TidyPlates_Neon"
local NO_MODE, TANK_MODE, DPS_MODE, PVP_MODE = 1, 2, 3, 4
local HEALTH_PCT, HEALTH_TOTAL, HEALTH_DEF = 2, 3, 4
local WIDGET_NONE, WIDGET_LINE = 1, 2

if not TidyPlatesNeonVariables then TidyPlatesNeonVariables = {} end
TidyPlatesNeonVariables = {Mode = 1, HealthText = 1, AutoHide = false, WidgetFunction = WIDGET_NONE, GlobalScale = 1}

------------------------------
-- Backup the Default Variables
------------------------------

local function copytable(original)
	local duplicate = {}
	for key, value in pairs(original) do
		if type(value) == "table" then duplicate[key] = copytable(value)
		else duplicate[key] = value end
	end
	return duplicate
end

local TidyPlatesNeonVariablesDefaults =copytable( TidyPlatesNeonVariables )

---------------
-- Target Widget
---------------
local targetwidgetimage = path.."\\Art\\NeonArt\\Neon_Select"
local function CreateTargetWidget(frame)
	local icon = frame:CreateTexture(nil, 'ARTWORK', frame)
	icon:SetTexture(targetwidgetimage)
	icon:SetWidth(128)
	icon:SetHeight(32)
	icon:Hide()
	icon.SetTarget = function (self, value) 
		if value then icon:Show() else icon:Hide() end 
	end
	return icon
end

---------------
-- Threat Functions
---------------

local threatfont = "Interface\\Addons\\TidyPlates_Neon\\Art\\arial.ttf"
local function SetFade(frame)
	if GetTime() > frame.FadeTime then
		frame:Hide()
		frame:SetScript("OnUpdate", nil)
	end
end

local function GetThreatSegment(threat)
	if not threat then return nil end
	if threat > 135 then return "HIGHTANK"
	elseif threat > 110 then return "MEDIUMTANK"
	elseif threat > 100 then return "LOWTANK"
	elseif threat > 85 then return "HIGHDPS"
	elseif threat > 55 then return "MEDIUMDPS"
	else return "LOWDPS"  end
end

local function GetRelativeThreat(unit)
	if not UnitExists(unit) then return end
	local _, group, size, index, unitid
	local playerthreat, leaderthreat, tempthreat, leadername = 0,0,0, nil
	_, _,  playerthreat, _, _ = UnitDetailedThreatSituation("player", unit)
	if HasPetUI() then _, _,  leaderthreat, _, _ = UnitDetailedThreatSituation("pet", unit); leadername = "pet" end
	if UnitInRaid("player") then group = "raid"; size = GetNumRaidMembers() - 1
	elseif UnitInParty("player") then group = "party"; size = GetNumPartyMembers()
	else group = nil end
	if group then
		for index = 1, size do
			unitid = group..index
			_, _, tempthreat , _, _ = UnitDetailedThreatSituation(unitid, unit)
			if tempthreat and tempthreat > leaderthreat then 
				leaderthreat = tempthreat 
				leadername = unitid
				end
		end
	end
	if playerthreat and leaderthreat then
		if playerthreat == 100 then return playerthreat + (100-leaderthreat), leadername
		else return playerthreat, leadername end
	else return end
end


---------------
-- Threat Line Widget
---------------

--local THREATWIDGET_LOW_COLOR = { r = .14, g = .75, b = 1}
local THREATWIDGET_LOW_COLOR = { r = 179/255, g = 246/255, b = 0}
local THREATWIDGET_HIGH_COLOR = {r = 1, g = .67, b = .14}
local threatlinepath = path.."\\Art\\ThreatLine\\"

local function UpdateThreatLineWidget(frame, unitid)
	if unitid then
		local threat, topholder = GetRelativeThreat(unitid) 
		--local threat, topholder = 75, "player"
		local threatcolor
		local middledot, outerdot = "Left", "Right"
		if threat and threat > 0 then
			local lineAnchor, dotAnchor, lineWidth = "RIGHT", "LEFT", 0
			-- Get Positions and Size
			if threat > 100 then
				-- While tanking
				lineWidth = ceil((threat - 100)/2)
				lineAnchor, dotAnchor = "LEFT", "RIGHT"
				--print("Tanking")
				threatcolor = THREATWIDGET_HIGH_COLOR
				frame.TargetText:SetText("")
			else 
				-- While NOT tanking
				lineWidth = ceil((100 - threat)/2)
				-- Tell who is
				--if topholder then 
				--	frame.TargetText:SetText(UnitName(topholder)) 
				--else frame.TargetText:SetText("") end
				--print("Not it!")
				threatcolor = THREATWIDGET_LOW_COLOR
				middledot, outerdot = "Right", "Left"
			end
				
				if topholder then 
					frame.TargetText:SetText(UnitName(topholder)) 
					--local _, class = UnitClass(topholder)
					--local classcolor = RAID_CLASS_COLORS[class]
					--frame.TargetText:SetTextColor(classcolor.r, classcolor.g, classcolor.b) 
				else frame.TargetText:SetText("") end
				
			-- Set Size
			frame.Line:ClearAllPoints()
			frame.Line:SetPoint(lineAnchor, frame.Middle, "CENTER")
			frame.Line:SetWidth(lineWidth)
			frame.Dot:ClearAllPoints()
			frame.Dot:SetPoint("CENTER", frame.Line, dotAnchor)
			-- Set Colors
			frame.Dot:SetVertexColor(threatcolor.r, threatcolor.g, threatcolor.b)
			frame.Line:SetVertexColor(threatcolor.r, threatcolor.g, threatcolor.b)
			frame.Middle:SetVertexColor(threatcolor.r, threatcolor.g, threatcolor.b)
			-- Set Textures
			frame.Middle:SetTexture(threatlinepath..middledot.."Dot")
			frame.Dot:SetTexture(threatlinepath..outerdot.."Dot")
			-- Set Fading
			frame:Show()
			frame.FadeTime = GetTime() + 3
			frame:SetScript("OnUpdate", SetFade)
		else frame:Hide() end
	elseif (GetTime() > frame.FadeTime) or (not InCombatLockdown()) then frame:Hide() end
end

local function CreateThreatLineWidget(parent, showname, hangtime, art)
	local frame = CreateFrame("Frame", nil, parent)
	frame:SetWidth(100)
	frame:SetHeight(24)
	-- Threat Center
	frame.Middle = frame:CreateTexture(nil, "OVERLAY")
	frame.Middle:SetPoint("CENTER")
	frame.Middle:SetWidth(32)
	frame.Middle:SetHeight(32)
	-- Threat Dot
	frame.Dot = frame:CreateTexture(nil, "OVERLAY")
	frame.Dot:SetWidth(32)
	frame.Dot:SetHeight(32)
	-- Threat Line
	frame.Line = frame:CreateTexture(nil, "OVERLAY")
	frame.Line:SetTexture(threatlinepath.."Threatline")
	frame.Line:SetWidth(32)
	frame.Line:SetHeight(32)
	-- Target Text
	frame.TargetText = frame:CreateFontString(nil, "OVERLAY")
	frame.TargetText:SetFont(threatfont, 9, "NONE")
	frame.TargetText:SetPoint("BOTTOM",frame.Dot,"TOP", 0, -15)
	frame.TargetText:SetWidth(50)
	frame.TargetText:SetHeight(20)
	-- Mechanics/Setup
	frame.FadeTime = 0
	frame:Hide()
	frame:RegisterEvent("CURSOR_UPDATE")
	frame:SetScript("OnEvent", function() TidyPlates:Update() end)
	frame.UpdateThreat = UpdateThreatLineWidget
	return frame
end

---------------
-- Text Helpers
---------------

local function valueToString(value)
	if value >= 1000000 then return format('%.1fm', value / 1000000)
	elseif value >= 1000 then return format('%.1fk', value / 1000)
	else return value end
end

---------------
-- Text Delegates
---------------

local function SpellTextDelegate(unit)
	local spellname
	if unit.isCasting then 
		spellname = UnitCastingInfo("target") or UnitChannelInfo("target")
		return spellname
	else return "" end
end

local function HealthTextDelegate(unit)
	if TidyPlatesNeonVariables.HealthText == HEALTH_PCT then
		if unit.health ~= unit.healthmax then 
			return "%"..ceil(100*(unit.health/unit.healthmax))
		else return nil end
	elseif TidyPlatesNeonVariables.HealthText == HEALTH_TOTAL then
		return valueToString(unit.health)
	elseif TidyPlatesNeonVariables.HealthText == HEALTH_DEF then
		if unit.health ~= unit.healthmax then return "-"..valueToString(unit.healthmax - unit.health) end
	else return "" end
end

---------------
-- Graphics Delegates
---------------
local classiconpath = path.."\\Art\\Class\\"
local function ArtDelegate(unit)
	if TidyPlatesNeonVariables.Mode == 4 then
		if unit.class and (unit.class ~= "UNKNOWN") then 
			return classiconpath..unit.class   
		end
	else return nil end
end		

local function ScaleDelegate(unit)
	local basescale = TidyPlatesNeonVariables.GlobalScale
	if InCombatLockdown() then
		-- Set Scale for Tank Mode
		if TidyPlatesNeonVariables.Mode == TANK_MODE and unit.reaction == "HOSTILE" and InCombatLockdown() then
			if (not unit.isElite) then basescale = basescale * .7 end
			if not unit.isInCombat then return basescale * .7 end			-- test
			if unit.threatSituation == "LOW" then return basescale * 1.2
			elseif unit.isCasting then return basescale
			elseif unit.threatSituation == "HIGH" then return basescale * 0.85 end
		-- Set Scale for DPS Mode
		elseif TidyPlatesNeonVariables.Mode == DPS_MODE then
			if (not unit.isElite) then basescale = basescale * .8 end
			if unit.threatSituation == "HIGH" then return basescale * 1.2
			elseif unit.isCasting then return basescale
			elseif unit.threatSituation == "LOW" then return basescale * .9 end
		elseif TidyPlatesNeonVariables.Mode == PVP_MODE then end
	end
	return basescale	
end

local function AlphaDelegate(unit)
	local basealpha = 1
	if unit.isCasting or unit.isMarked then return 1 end
	
	if InCombatLockdown() then
		-- Mode-Specific Filters
		if TidyPlatesNeonVariables.Mode == TANK_MODE then
			if (not unit.isElite) then basealpha = .6 end
			if unit.threatSituation == "HIGH" then return basealpha * 0.85 end
		elseif TidyPlatesNeonVariables.Mode == DPS_MODE then
			if (not unit.isElite) then basealpha = .8 end
			if unit.threatSituation == "LOW" then return basealpha * 0.85 end
		end
		-- Basic Mode Filters
		if TidyPlatesNeonVariables.Mode > NO_MODE then
			if unit.reaction == "NEUTRAL" and unit.threatSituation == "LOW" then return 0 
			elseif not unit.isInCombat then return basealpha * .5				-- test
			else return basealpha end
		end
	end
	return 1
end

-- local = ( 0, .46, 1 }
local function HealthColorDelegate(unit)
	if InCombatLockdown() then
		--if TidyPlatesNeonVariables.Mode > NO_MODE then
			--if unit.reaction == "HOSTILE" and unit.threatSituation ~= "HIGH" then return .36, 1, 0 
			if unit.reaction == "HOSTILE" and unit.threatSituation == "LOW" then return 0, .46, 1 
			else return unit.red, unit.green, unit.blue end
			
		--end
	end
	return unit.red, unit.green, unit.blue
end


---------------
-- Adjust Vertical Position
---------------
local verticaladjust = 10

---------------
-- Widgets
---------------

local widgetadjust = -1
local function OnInitializeDelegate(plate)
	if TidyPlatesNeonVariables.WidgetFunction == WIDGET_LINE then
		plate.ThreatLineFrame = CreateThreatLineWidget(plate)
		plate.ThreatLineFrame:SetPoint("CENTER", plate, 0, 5+verticaladjust+widgetadjust)
	end
	plate.targetImage = CreateTargetWidget(plate)
	plate.targetImage:SetPoint("CENTER", plate, 0, verticaladjust)
 end

local function OnUpdateDelegate(plate, unit)
	if TidyPlatesNeonVariables.WidgetFunction == WIDGET_LINE then
		if not plate.ThreatLineFrame then OnInitializeDelegate(plate) end
		local unitid = nil
		if unit.isMouseover then plate.ThreatLineFrame:UpdateThreat("mouseover")
		elseif unit.alpha == 1 and UnitExists("target") then plate.ThreatLineFrame:UpdateThreat("target")
		else plate.ThreatLineFrame:UpdateThreat(nil) end
	end
	
	if not plate.targetImage then OnInitializeDelegate(plate) end
	plate.targetImage:SetTarget(unit.isTarget)

end


---------------
-- Theme definition (Just uses the default art)
---------------
local artpath = "Interface\\Addons\\TidyPlates_Neon\\Art\\NeonArt\\"
local castadjust = -21
local nameadjust = -8			-- For Below
--local nameadjust = 8			-- For ABOVE

local themedata = {
	healthborder = {
		texture		 =				artpath.."Neon_HealthOverlay",
		glowtexture =					artpath.."Neon_Highlight",
		elitetexture =					artpath.."Neon_HealthOverlay",
		width = 128,
		height = 32,
		y = 0+verticaladjust,
	},

	healthbar = {
		texture =					 artpath.."Neon_Bar",
		width = 100,
		height = 32,
		x = 0,
		y = 0+verticaladjust,
	},

	castborder = {
		texture =					artpath.."Neon_CastOverlayBlue",
		width = 128,
		height = 32,
		x = 0,
		y = castadjust+verticaladjust,
	},
	
	castnostop = {
		texture =					artpath.."Neon_CastOverlayRed",
		width = 128,
		height = 32,
		x = 0,
		y = castadjust+verticaladjust,
	},
	
	
	castbar = {
		texture =					 artpath.."Neon_Bar",
		width = 100,
		height = 32,
		x = 0,
		y = castadjust+verticaladjust,
		anchor = "CENTER",
		orientation = "HORIZONTAL",
	},
	
	threatborder = {
		texture =				artpath.."Neon_Empty",
		elitetexture =			artpath.."Neon_Empty",
		width = 8,
		height = 8,
	},
	
	spellicon = {
		width = 17,
		height = 17,
		x = 26,
		y = -3+castadjust+verticaladjust,
		anchor = "CENTER",
	},
	
	name = {
		size = 11,
		width = 150,
		height = 11,
		x = 0,
		y = nameadjust+verticaladjust,
		align = "CENTER",
		anchor = "CENTER",
		vertical = "CENTER",
		shadow = true,
	},
	
	specialText = {
		size = 11,
		width = 150,
		height = 11,
		x = 26,
		y = -19+castadjust+verticaladjust,
		align = "CENTER",
		anchor = "CENTER",
		vertical = "CENTER",
		shadow = true,
	},
	
	options = {
		showName = true,
		showLevel = false,
		showSpecialText = true,
		showSpecialArt = true,
		useCustomHealthbarColor = true,
		forceAlpha = true,
	},
	
	SetSpecialText = SpellTextDelegate,
	SetSpecialArt = ArtDelegate,
	SetScale = ScaleDelegate,
	SetAlpha = AlphaDelegate,
	OnUpdate = OnUpdateDelegate,
	OnInitialize = OnInitializeDelegate,
	SetHealthbarColor = HealthColorDelegate, 
	
}

TidyPlatesThemeList["Neon"] = themedata

------------------------------------------------------------------
-- Panel
------------------------------------------------------------------

-- Main Panel
local panel = PanelHelpers:CreatePanelFrame( "TidyPlatesNeonInterfaceOptions", "Neon Theme" )
panel.parent = "Tidy Plates"

---- MODE 
panel.radiobuttonlabel = panel:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
panel.radiobuttonlabel:SetPoint("TOPLEFT", panel, "TOPLEFT", 35, -65)
panel.radiobuttonlabel:SetText("Display Mode:")
local radiolist = {"Default: |cFFFFFFFF No Scaling or extra art.",
		"Tank: |cFFFFFFFF Reduces scale and opacity of units that are attacking you. ",
		"Damage: |cFFFFFFFF Increases scale and opacity of units that are attacking you.", 
		"PVP: |cFFFFFFFF  Shows enemy class icon on the nameplate. "}
panel.radiobuttons = PanelHelpers:CreateRadioButtons("TidyPlatesNeon_ModeRadio", panel, 4, 1, 20, radiolist)
panel.radiobuttons[1]:SetPoint("TOPLEFT", panel.radiobuttonlabel, 0, -17)

---- WIDGET
panel.dropdownmenulabel = panel:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
panel.dropdownmenulabel:SetPoint("TOPLEFT", panel, "TOPLEFT", 35, -225)
panel.dropdownmenulabel:SetText("Widget Function:")
local dropmenu = { 
				{ text = "None", notCheckable = 1 },
				{ text = "Threat Line", notCheckable = 1 } ,
			}
panel.dropdown = PanelHelpers:CreateDropdownFrame("TidyPlatesNeon_HealthTextDropdown", panel, dropmenu, 1)
panel.dropdown:SetPoint("TOPLEFT", panel.dropdownmenulabel, 0, -17)
panel.dropdown:SetValue(2)

--  SCALE
panel.Scalelabel = panel:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
panel.Scalelabel:SetPoint("TOPLEFT", panel.dropdownmenulabel, "BOTTOMLEFT", 0, -50)
panel.Scalelabel:SetText("Nameplate Scale:")
panel.Scale = PanelHelpers:CreateSliderFrame("TidyPlatesNeon_ScaleSlider", panel)
panel.Scale:SetPoint("TOPLEFT", panel.Scalelabel, "BOTTOMLEFT", 5, -7)
panel.Scale:SetMinMaxValues(.5, 1.5)
panel.Scale:SetValue(1)
panel.Scale:SetValueStep(.1) 
panel.Scale.Low:SetText("50%")
panel.Scale.High:SetText("150%")
--]]

-- APPLY BUTTON
panel.apply = CreateFrame("Button", "TidyPlatesNeon_ApplyButton", panel, "UIPanelButtonTemplate2")
--panel.apply:SetPoint("TOPLEFT", panel.Scale, "BOTTOMLEFT", 0, -50)
panel.apply:SetPoint("BOTTOMRIGHT", panel, -15, 15)
panel.apply:SetText("Apply")
panel.apply:SetWidth(80)

-- Functions
panel.okay = function () 
	TidyPlatesNeonVariables.Mode = panel.radiobuttons:GetChecked()
	TidyPlatesNeonVariables.WidgetFunction = panel.dropdown.Value
	TidyPlatesNeonVariables.GlobalScale = panel.Scale:GetValue()
	TidyPlates:ForceUpdate()
end

-- APPLY BUTTON Script
panel.apply:SetScript("OnClick",panel.okay)

panel.refresh = function ()
	panel.radiobuttons:SetChecked(TidyPlatesNeonVariables.Mode)
	panel.dropdown:SetValue(TidyPlatesNeonVariables.WidgetFunction)
	panel.Scale:SetValue(TidyPlatesNeonVariables.GlobalScale)
end

panel:SetScript("OnEvent", function(self, event, ...) InterfaceOptions_AddCategory(panel) end)
panel:RegisterEvent("PLAYER_LOGIN")

--------------------------------------------  Slash Commands  --------------------------------------------
TidyPlatesNeonSlashCommands = {}
TidyPlatesNeonSlashCommands.tank = function() print("Tidy Plates Neon: Tank Mode Activated"); TidyPlatesNeonVariables.Mode = TANK_MODE end
TidyPlatesNeonSlashCommands.dps = function() print("Tidy Plates Neon: DPS Mode Activated"); TidyPlatesNeonVariables.Mode = DPS_MODE end
TidyPlatesNeonSlashCommands.pvp = function() print("Tidy Plates Neon: PVP Mode Activated"); TidyPlatesNeonVariables.Mode = PVP_MODE end
TidyPlatesNeonSlashCommands.reset = function() print("Tidy Plates Neon: Variables Reset"); TidyPlatesNeonVariables = copytable( TidyPlatesNeonVariablesDefaults ) end
--------------------------------------------  Slash Commands  --------------------------------------------
function slash_TidyNeon(arg)
	if type(TidyPlatesNeonSlashCommands[arg]) == 'function' then 
		TidyPlatesNeonSlashCommands[arg]() 
		TidyPlates:ForceUpdate()
	else InterfaceOptionsFrame_OpenToCategory(panel);ChatFrame1:AddMessage('Tidy Plates Neon Commands;  /tidyneon {tank, dps, pvp}') end
end

SLASH_TIDYNEON1 = '/tidyneon'
SlashCmdList['TIDYNEON'] = slash_TidyNeon;










